/*
 * Copyright (C) 2023 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include <span>
#include <wtf/ThreadSafeWeakPtr.h>

namespace WebCore {

class ReadableStreamSource;
class WebTransportBidirectionalStream;
class WebTransportSendStream;
class WebTransportSessionClient;
class WritableStreamSink;

struct WebTransportBidirectionalStreamConstructionParameters;

class WEBCORE_EXPORT WebTransportSession {
public:
    virtual ~WebTransportSession();
    virtual void ref() const = 0;
    virtual void deref() const = 0;

    virtual void sendDatagram(std::span<const uint8_t>, CompletionHandler<void()>&&) = 0;
    virtual void createOutgoingUnidirectionalStream(CompletionHandler<void(RefPtr<WritableStreamSink>&&)>&&) = 0;
    virtual void createBidirectionalStream(CompletionHandler<void(std::optional<WebTransportBidirectionalStreamConstructionParameters>&&)>&&) = 0;
    virtual void terminate(uint32_t, CString&&) = 0;

    void attachClient(WebTransportSessionClient&);
protected:
    ThreadSafeWeakPtr<WebTransportSessionClient> m_client;
};

}
