/*
 * Copyright (C) 2021-2023 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(GPU_PROCESS)
#include "RemoteImageBufferSet.h"

#if ENABLE(RE_DYNAMIC_CONTENT_SCALING)
#include "ArgumentCoders.h" // NOLINT
#endif
#include "Decoder.h" // NOLINT
#include "HandleMessage.h" // NOLINT
#include "IPCEvent.h" // NOLINT
#include "RemoteImageBufferSetMessages.h" // NOLINT
#include "WebCoreArgumentCoders.h" // NOLINT
#include <WebCore/DestinationColorSpace.h> // NOLINT
#if ENABLE(RE_DYNAMIC_CONTENT_SCALING)
#include <WebCore/DynamicContentScalingDisplayList.h> // NOLINT
#endif
#include <WebCore/FloatSize.h> // NOLINT
#include <WebCore/ImageBufferBackend.h> // NOLINT
#include <WebCore/RenderingMode.h> // NOLINT
#if ENABLE(RE_DYNAMIC_CONTENT_SCALING)
#include <optional> // NOLINT
#endif

#if ENABLE(IPC_TESTING_API)
#include "JSIPCBinding.h"
#endif

namespace WebKit {

void RemoteImageBufferSet::didReceiveStreamMessage(IPC::StreamServerConnection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::RemoteImageBufferSet::UpdateConfiguration::name())
        return IPC::handleMessage<Messages::RemoteImageBufferSet::UpdateConfiguration>(connection.protectedConnection(), decoder, this, &RemoteImageBufferSet::updateConfiguration);
    if (decoder.messageName() == Messages::RemoteImageBufferSet::SetFlushSignal::name())
        return IPC::handleMessage<Messages::RemoteImageBufferSet::SetFlushSignal>(connection.protectedConnection(), decoder, this, &RemoteImageBufferSet::setFlushSignal);
    if (decoder.messageName() == Messages::RemoteImageBufferSet::Flush::name())
        return IPC::handleMessage<Messages::RemoteImageBufferSet::Flush>(connection.protectedConnection(), decoder, this, &RemoteImageBufferSet::flush);
#if ENABLE(RE_DYNAMIC_CONTENT_SCALING)
    if (decoder.messageName() == Messages::RemoteImageBufferSet::DynamicContentScalingDisplayList::name())
        return IPC::handleMessageSynchronous<Messages::RemoteImageBufferSet::DynamicContentScalingDisplayList>(connection, decoder, this, &RemoteImageBufferSet::dynamicContentScalingDisplayList);
#endif
    UNUSED_PARAM(decoder);
    UNUSED_PARAM(connection);
#if ENABLE(IPC_TESTING_API)
    if (connection.protectedConnection()->ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled stream message %s to %" PRIu64, IPC::description(decoder.messageName()), decoder.destinationID());
}

} // namespace WebKit

#if ENABLE(IPC_TESTING_API)

namespace IPC {

template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::RemoteImageBufferSet_UpdateConfiguration>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::RemoteImageBufferSet::UpdateConfiguration::Arguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::RemoteImageBufferSet_SetFlushSignal>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::RemoteImageBufferSet::SetFlushSignal::Arguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::RemoteImageBufferSet_Flush>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::RemoteImageBufferSet::Flush::Arguments>(globalObject, decoder);
}
#if ENABLE(RE_DYNAMIC_CONTENT_SCALING)
template<> std::optional<JSC::JSValue> jsValueForDecodedMessage<MessageName::RemoteImageBufferSet_DynamicContentScalingDisplayList>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::RemoteImageBufferSet::DynamicContentScalingDisplayList::Arguments>(globalObject, decoder);
}
template<> std::optional<JSC::JSValue> jsValueForDecodedMessageReply<MessageName::RemoteImageBufferSet_DynamicContentScalingDisplayList>(JSC::JSGlobalObject* globalObject, Decoder& decoder)
{
    return jsValueForDecodedArguments<Messages::RemoteImageBufferSet::DynamicContentScalingDisplayList::ReplyArguments>(globalObject, decoder);
}
#endif

}

#endif


#endif // ENABLE(GPU_PROCESS)
